/********************************************************
  Xpress-BCL C++ Example Problems
  ===============================

  file xbdelvr.cxx
  `````````````````
  Transportation problem.

  (c) 2008-2024 Fair Isaac Corporation
      author: S.Heipcke, Jan. 2000, rev. Mar. 2011
********************************************************/

#include <iostream>
#include <cstdio>
#include "xprb_cpp.h"

using namespace std;
using namespace ::dashoptimization;

#define NSupp 10                  /* Number of suppliers */
#define NCust 7                   /* Number of customers */
#define MaxArcs 100               /* Max. num. of non-zero cost values */

#define VANFILE XPRBDATAPATH "/delivery/ifvan.dat"    /* Van data file */
#define COSTFILE XPRBDATAPATH "/delivery/cost.dat"    /* Cost data file */

/****DATA****/
/* Supplier:      London  Luton  B'ham Bristl  Derby Stckpt   York */
double SUPPLY[] = {140.0, 600.0,  50.0,  10.0, 400.0, 200.0,  20.0,
/* Supplier: Derby  Soton Scnthp */
             90.0,  30.0,  12.0};
/* Customer:      London Livpol Doncst   York   Hull Manchr Shffld */
double DEMAND[] = {123.3,  56.4,  17.1, 192.8, 310.0,  47.0,  86.0};

double COST[NSupp][NCust];        /* Cost per supplier-customer pair */

double IFVAN[NSupp][NCust];       /* Non-zero if route uses vans instead 
                                     of lorries */
double VANCAP=40.0;               /* Capacity on routes that use vans */

/***********************************************************************/

void modDelivery(XPRBprob &p)
{
 XPRBexpr lobj, lc;
 int s,c;
 XPRBvar x[NSupp][NCust];
 
/****VARIABLES****/
 for(s=0;s<NSupp;s++)
  for(c=0; c<NCust; c++)
   x[s][c] = p.newVar(XPRBnewname("x_s%d",s));
     
/****OBJECTIVE****/
 for(s=0;s<NSupp;s++)             /* Objective: Minimize total cost */
  for(c=0; c<NCust; c++)
   lobj += COST[s][c]*x[s][c];
 p.setObj(p.newCtr("OBJ", lobj)); /* Set objective function */ 

/****CONSTRAINTS****/  
 for(c=0; c<NCust; c++)           /* Satisfy demand of each customer */
 {
  lc=0;
  for(s=0;s<NSupp;s++)  lc += x[s][c];
  p.newCtr("Demand", lc >= DEMAND[c]);
 }
        
 for(s=0;s<NSupp;s++)             /* Keep within supply at each supplier*/
 {
  lc=0;
  for(c=0; c<NCust; c++)  lc+= x[s][c];
  p.newCtr("Supply", lc <= SUPPLY[s]);
 }  
 
/****BOUNDS****/
 for(s=0;s<NSupp;s++)
  for(c=0; c<NCust; c++)
   if(IFVAN[s][c]!=0) x[s][c].setUB(VANCAP);

/****SOLVING + OUTPUT****/
 p.exportProb(XPRB_MPS,"delivery");   /* Write out an MPS file */

 p.setSense(XPRB_MINIM);           /* Set objective sense to minimization */
 p.lpOptimize("");                 /* Solve the LP-problem */
 cout << "Objective: " << p.getObjVal() << endl;  /* Get objective value */
    
 for(s=0;s<NSupp;s++)              /* Print out the solution values */
  for(c=0; c<NCust; c++)
   cout << x[s][c].getName() << ":" << x[s][c].getSol() << " ";
 cout << endl;    
}

/***********************************************************************/

    /**** Read data from files ****/
void readData()
{
 FILE *datafile;
 int s,c;  

/* Initialize data tables to 0: in the van data file some entries that are 
 * zero are simply left out, but the function XPRBreadarrline only initializes 
 * those elements to 0 that have been read, e.g. a data line ",,," results
 * in the first 4 elements of the array to be set to zero, even if the
 * maximum number of elements to be read (=last parameter of XPRBreadarrline)
 * has a much larger value */
 for(s=0;s<NSupp;s++)
  for(c=0; c<NCust; c++)
  {
   COST[s][c] = 0;
   IFVAN[s][c] = 0;
  }  
        /* Read the demand data file */
 datafile=fopen(COSTFILE,"r");
 for(s=0;s<NSupp;s++)
  XPRBreadarrlinecb(XPRB_FGETS, datafile, 99, "g,", COST[s], NCust);
 fclose(datafile);

        /* Read the van data file */
 datafile=fopen(VANFILE,"r");
 for(s=0;s<NSupp;s++)
  XPRBreadarrlinecb(XPRB_FGETS, datafile, 99,"g,", IFVAN[s], NCust);
 fclose(datafile);
}

/***********************************************************************/

int main(int argc, char **argv)
{
 XPRBprob p("Delivery"); /* Initialize a new problem in BCL */
 readData();             /* Data input from file */
 modDelivery(p);         /* Problem formulation and solution */

 return 0;
} 
